param($ComputerName,$DomainUsername,$DelPwd,$tejasFQDN,$info)

<#################### STAGES ########################
1 = Host Group Created
2 = New-SCLogicalNetwork created
3 = New-SCSubnetVLan created
4 = New-SCLogicalNetworkDefinition created
5 = New-SCVMNetwork created
6 = New-SCVMSubnet created
7 = New-SCDefaultGateway created
8 = New-SCStaticIPAddressPool created
9 = New-SCNativeUplinkPortProfile created
10 = New-SCLogicalSwitch created
11 = New-SCUplinkPortProfileSet created
12 = New-SCLogicalSwitchVirtualNetworkAdapter created
################### STAGES ##########################>

###################### VARILABLES ###########################
<#$ComputerName="sharuscvmm32.mydomain.com"
$DomainUsername="mydomain\Administrator"
$DelPwd="Dell_123$"#>
$info = $info -replace "\\n\\r", " "

$switchName=$info.split("$")[0]
$hostGroupname=$info.split("$")[1]
$infoInJson=$info.split("$")[2]
write-host $hostGroupname
write-host $infoInJson
<#
$infoInJson= '{
        "mgmtNetworkName":"Management",
        "mgmtSubnet":"100.96.34.0/24",
        "mgmtVlanId":"0",
        "mgmtStartIp":"100.96.34.60",
        "mgmtEndIp":"100.96.34.70",
        "DNSServer":"100.96.32.4",
        "DNSSuffix":"S2D.com",
        "Gateway":"100.96.34.4",
        "smb1NetworkName":"Storage1",
        "smb1Subnet":"100.96.34.30/24",
        "smb1VlanId":"103",
        "smb1StartIp":"100.96.34.30",
        "smb1EndIp":"100.96.34.40",
        "smb2NetworkName":"Storage2",
        "smb2Subnet":"100.96.34.50/24",
        "smb2VlanId":"104",
        "smb2StartIp":"100.96.34.60",
        "smb2EndIp":"100.96.34.70"
    }'
#>
#$hostGroupname="S2D Hosts"
#$WindowsCredentialProfile= "OMIMSSC_AcessCP"
#$switchName="TheSwitch"

$StagesOfDevlopment=@()

###################### VARILABLES ###########################


$returnval = @()

###################### FUNCTIONS #############################

# Create Credetial Object 
Function Createcredobj($username, $passwd)
{

    try
    {
		$secpasswd = ConvertTo-SecureString  $passwd -AsPlainText -Force
        $mycredsobj = New-Object System.Management.Automation.PSCredential ($username, $secpasswd)
        #$Error
    }
    catch
    {
        $mycredsobj = $null
    }

    return $mycredsobj

}

###################### FUNCTIONS #############################

#import SCVMM related cmdlets
import-module virtualmachinemanager

try{
    
    $resource=$infoInJson | Out-String | ConvertFrom-Json
    write-host $resource
    #Connecting to SCVMM
	write-host "connecting to SCVMM "+$ComputerName
    #Creating Credential
    $scvmmcred= Createcredobj $DomainUsername $DelPwd
    $vmm=Get-SCVMMServer -ComputerName $ComputerName -Credential $scvmmcred

    if($vmm -ne $null){
        #Connection to SCVMm Successfull
        Write-Host "SCVMM found on the system, Getting the hostGroup "+$hostGroupname

        #Getting a specific Host Group name, if it doesnt exist, we will create the host group
        $hostGroup=Get-SCVMHostGroup -VMMServer $vmm -Name $hostGroupname
        if($hostGroup -eq $null){
            Write-Host "Failed to get the Host Group "+$hostGroupname
            $failedLogicalNetwork="Failed to get the Host Group "+$hostGroupname
			throw $failedLogicalNetwork
        }
        

        #creating Vlan-Based specific Logical Networks
        Write-Host "creating Vlan-Based specific Logical Networks"
        $mgmtName=$resource.mgmtNetworkName+"_"+$switchName
        $mgmtLogicalNetwork=New-SCLogicalNetwork -Name $mgmtName -LogicalNetworkDefinitionIsolation $true -EnableNetworkVirtualization $false -UseGRE $false -IsPVLAN $false -VMMServer $vmm
        $StagesOfDevlopment=$StagesOfDevlopment+'2'

        $smb1Name=$resource.smb1NetworkName+"_"+$switchName
        $smb1LogicalNetwork=New-SCLogicalNetwork -Name $smb1Name -LogicalNetworkDefinitionIsolation $true -EnableNetworkVirtualization $false -UseGRE $false -IsPVLAN $false -VMMServer $vmm

        $smb2Name=$resource.smb2NetworkName+"_"+$switchName
        $smb2LogicalNetwork=New-SCLogicalNetwork -Name $smb2Name -LogicalNetworkDefinitionIsolation $true -EnableNetworkVirtualization $false -UseGRE $false -IsPVLAN $false -VMMServer $vmm 

        #creating the SubnetVlan
        Write-Host "creating the subnet Vlan"
        
        $mgmtSubnetVLAN=New-SCSubnetVLan -Subnet $resource.mgmtSubnet -VLanID $resource.mgmtVlanId -VMMServer $vmm
        $StagesOfDevlopment=$StagesOfDevlopment+'3'
        $smb1SubnetVLAN=New-SCSubnetVLan -Subnet $resource.smb1Subnet -VLanID $resource.smb1VlanId -VMMServer $vmm
        $smb2SubnetVLAN=New-SCSubnetVLan -Subnet $resource.smb2Subnet -VLanID $resource.smb2VlanId -VMMServer $vmm

        #creating logicalnetwork defination
        Write-Host "creating logical network defination"
        $mgmtDefination=New-SCLogicalNetworkDefinition -Name "$($mgmtName)_0" -LogicalNetwork $mgmtLogicalNetwork -VMHostGroup $hostGroup -SubnetVLan $mgmtSubnetVLAN -RunAsynchronously -VMMServer $vmm
        $StagesOfDevlopment=$StagesOfDevlopment+'4'
        $smb1Defination=New-SCLogicalNetworkDefinition -Name "$($smb1Name)_0" -LogicalNetwork $smb1LogicalNetwork -VMHostGroup $hostGroup -SubnetVLan $smb1SubnetVLAN -RunAsynchronously -VMMServer $vmm
        $smb2Defination=New-SCLogicalNetworkDefinition -Name "$($smb2Name)_0" -LogicalNetwork $smb2LogicalNetwork -VMHostGroup $hostGroup -SubnetVLan $smb2SubnetVLAN -RunAsynchronously -VMMServer $vmm

        #Create VM Network
        Write-Host "Create SCVMM Network"
        $mgmtVmNetwork=New-SCVMNetwork -Name $mgmtName -LogicalNetwork $mgmtLogicalNetwork -IsolationType "VLANNetwork" -VMMServer $vmm
        $StagesOfDevlopment=$StagesOfDevlopment+'5'
        $smb1VmNetwork=New-SCVMNetwork -Name $smb1Name -LogicalNetwork $smb1LogicalNetwork -IsolationType "VLANNetwork" -VMMServer $vmm
        $smb2VmNetwork=New-SCVMNetwork -Name $smb2Name -LogicalNetwork $smb2LogicalNetwork -IsolationType "VLANNetwork" -VMMServer $vmm

        #Create VM Subnet
        write-host "create SCVMM Subnet"
        $mgmtSubnet=New-SCVMSubnet -Name "$($mgmtName)_Subnet_0" -LogicalNetworkDefinition $mgmtDefination -SubnetVLan $mgmtSubnetVLAN -VMNetwork $mgmtVmNetwork -VMMServer $vmm
        $StagesOfDevlopment=$StagesOfDevlopment+'6'
        $smb1Subnet=New-SCVMSubnet -Name "$($smb1Name)_Subnet_0" -LogicalNetworkDefinition $smb1Defination -SubnetVLan $smb1SubnetVLAN -VMNetwork $smb1VmNetwork -VMMServer $vmm
        $smb2Subnet=New-SCVMSubnet -Name "$($smb2Name)_Subnet_0" -LogicalNetworkDefinition $smb2Defination -SubnetVLan $smb2SubnetVLAN -VMNetwork $smb2VmNetwork -VMMServer $vmm
      
        #validating if the Logical network got created or not.
        write-host "validating if the Logical network got created or not "
        $ismgmtLogicalNetworkCreated=Get-SCLogicalNetwork -Name $mgmtName -VMMServer $vmm
        $issmb1LogicalNetworkCreated=Get-SCLogicalNetwork -Name $smb1Name -VMMServer $vmm
        $issmb2LogicalNetworkCreated=Get-SCLogicalNetwork -Name $smb2Name -VMMServer $vmm
       
        $failedLogicalNetwork=''

        if($ismgmtLogicalNetworkCreated -eq $null){
            Write-Host "Failed to create Mgmt Logical Network"
            $failedLogicalNetwork="Failed to create Mgmt Logical Network"
        }
        if($issmb1LogicalNetworkCreated -eq $null){
            Write-Host "Failed to create Smb1 Logical Network"
            $failedLogicalNetwork=$failedLogicalNetwork+ "Failed to create Smb1 Logical Network"
        }
        if($issmb2LogicalNetworkCreated -eq $null){
            Write-Host "Failed to create Smb2 Logical Network"
            $failedLogicalNetwork=$failedLogicalNetwork+ "Failed to create Smb2 Logical Network"
        }

        write-host "failedLogicalNetwork "$failedLogicalNetwork
        if(!([string]::IsNullorEmpty($failedLogicalNetwork))){
           	       
	       <# $returnval=$returnval+ "FAIL"
	        $returnval=$returnval+$failedLogicalNetwork

            return $returnval#>
            throw $failedLogicalNetwork
        }

        #creating Gateway in SCVMM
        Write-Host "craeting Gateway in SCVMM"
        $gateway = @()
        $gateway += New-SCDefaultGateway -IPAddress $resource.Gateway -Automatic -VMMServer $vmm
        $StagesOfDevlopment=$StagesOfDevlopment+'7'

        #creating IP POOl for DNS details given
        Write-Host "creating Ip Pool for the DNS details given"
        $poolName="Mgmt_"+$switchName+"_IPPool"
        New-SCStaticIPAddressPool -Name $poolName -LogicalNetworkDefinition $mgmtDefination -Subnet $resource.mgmtSubnet -IPAddressRangeStart $resource.mgmtStartIp -IPAddressRangeEnd $resource.mgmtEndIp -DefaultGateway $gateway -DNSServer $resource.DNSServer -DNSSuffix "" -DNSSearchSuffix $resource.DNSSuffix -RunAsynchronously -VMMServer $vmm|Out-Null

         $StagesOfDevlopment=$StagesOfDevlopment+'8'

         #creating UPP
         Write-Host "Creating UPP"
         $definition = @($mgmtDefination,$smb1Defination,$smb2Defination)
         $uplinkPortProfileName=$switchName+"_UPP"
         $nativeUppVar=New-SCNativeUplinkPortProfile -Name $uplinkPortProfileName -Description "Uplink Port profile created from OMIMSSC" -LogicalNetworkDefinition $definition -EnableNetworkVirtualization $false -LBFOLoadBalancingAlgorithm "HostDefault" -LBFOTeamMode "SwitchIndependent" -RunAsynchronously -VMMServer $vmm
         $StagesOfDevlopment=$StagesOfDevlopment+'9'
          
          write-host "UPP "$nativeUppVar       

         #validating if UPP was created
         $isUPPCreated=Get-SCNativeUplinkPortProfile -Name $uplinkPortProfileName -VMMServer $vmm
         if($nativeUppVar -eq $null){
            write-Host "Unable to create UPP"
           <# $returnval=$returnval+ "FAIL"
	        $returnval=$returnval+"Failed while creating UPP"

            return $returnval #>
            throw "Failed while creating UPP"
         }

         #creating logical Switch
         Write-Host "Creating Logical Switch"
         $logicalSwitch = New-SCLogicalSwitch -Name $switchName -Description "Logical Switch created by OMIMSSC" -EnableSriov $false -SwitchUplinkMode "EmbeddedTeam" -MinimumBandwidthMode "Weight" -VMMServer $vmm
         $StagesOfDevlopment=$StagesOfDevlopment+'10'

         $uppSetVar = New-SCUplinkPortProfileSet -Name $uplinkPortProfileName -LogicalSwitch $logicalSwitch -NativeUplinkPortProfile $nativeUppVar -RunAsynchronously -VMMServer $vmm
         $StagesOfDevlopment=$StagesOfDevlopment+'11'

         # Get Static IP Address Pool
         Write-Host "Creating SCLogicalSwitchVirtualNetworkAdapter"
         $ipV4Pool = Get-SCStaticIPAddressPool -Name $poolName -VMMServer $vmm
         New-SCLogicalSwitchVirtualNetworkAdapter -Name $mgmtName -UplinkPortProfileSet $uppSetVar -RunAsynchronously -VMNetwork $mgmtVmNetwork -VMSubnet $mgmtSubnet -IsUsedForHostManagement $true -InheritsAddressFromPhysicalNetworkAdapter $false -IPv4AddressType "Static" -IPv6AddressType "Dynamic" -IPv4AddressPool $ipV4Pool -VMMServer $vmm|Out-Null
         $StagesOfDevlopment=$StagesOfDevlopment+'12'
         New-SCLogicalSwitchVirtualNetworkAdapter -Name $smb1Name -UplinkPortProfileSet $uppSetVar -RunAsynchronously -VMNetwork $smb1VmNetwork -VMSubnet $smb1Subnet -IsUsedForHostManagement $false -InheritsAddressFromPhysicalNetworkAdapter $false -IPv4AddressType "Dynamic" -IPv6AddressType "Dynamic" -Verbose -VMMServer $vmm|Out-Null
         New-SCLogicalSwitchVirtualNetworkAdapter -Name $smb2Name -UplinkPortProfileSet $uppSetVar -RunAsynchronously -VMNetwork $smb2VmNetwork -VMSubnet $smb2Subnet -IsUsedForHostManagement $false -InheritsAddressFromPhysicalNetworkAdapter $false -IPv4AddressType "Dynamic" -IPv6AddressType "Dynamic" -Verbose -VMMServer $vmm|Out-Null

         #validate Logical Switch creation
         $isLogicalNetworkReady=Get-SCLogicalSwitch -Name $switchName -VMMServer $vmm

         if($isLogicalNetworkReady -ne $null){
            write-host "Logical switch created successfully"

            $returnval=$returnval+ "SUCCESS"
	        $returnval=$returnval+"Logical Switch created successfully."

            return $returnval
            
         }

    }else{
        #Connection to SCVMM failed
        write-host "SCVMM not found on the system"
        
           <# $returnval=$returnval+ "FAIL"
	        $returnval=$returnval+"SCVMM not found on the system"#>

            throw "SCVMM not found on the system"
    }

}
catch{
	write-host "Exception occured "+$_.Exception.Message
	$errmsg="Exception Occured"+$_.Exception.Message
	$returnval=$returnval+ "FAIL"
	$returnval=$returnval+$errmsg
	

    if($StagesOfDevlopment.Count -gt 0){
        #Deleting the Vm Network Formed so far, Rollback logic in case of exception in the flow
        try{
              $resource=$infoInJson | ConvertFrom-Json
    
                #Connecting to SCVMM
	            write-host "connecting to SCVMM "+$ComputerName
                #Creating Credential
                $scvmmcred= Createcredobj $DomainUsername $DelPwd
                $vmm=Get-SCVMMServer -ComputerName $ComputerName -Credential $scvmmcred

                $mgmtName=$resource.mgmtNetworkName+"_"+$switchName
                $smb1Name=$resource.smb1NetworkName+"_"+$switchName
                $smb2Name=$resource.smb2NetworkName+"_"+$switchName

                $uplinkPortProfileName=$switchName+"_UPP"
                $poolName="Mgmt_"+$switchName+"_IPPool"

                Write-Host " the Stages of development are : "+$StagesOfDevlopment
                if($StagesOfDevlopment.Contains('12')){
                    Write-Host "Removing SC Logical Switch Network Adaptor"
                    #remove everything
                    $mgmtLogicalSwitchVirtualNetworkAdaptor=Get-SCLogicalSwitchVirtualNetworkAdapter -Name $mgmtName -VMMServer $vmm
                    $smb1LogicalSwitchVirtualNetworkAdaptor=Get-SCLogicalSwitchVirtualNetworkAdapter -Name $smb1Name -VMMServer $vmm
                    $smb2LogicalSwitchVirtualNetworkAdaptor=Get-SCLogicalSwitchVirtualNetworkAdapter -Name $smb2Name -VMMServer $vmm

                    if($mgmtLogicalSwitchVirtualNetworkAdaptor -ne $null){Remove-SCLogicalSwitchVirtualNetworkAdapter -LogicalSwitchVirtualNetworkAdapter $mgmtLogicalSwitchVirtualNetworkAdaptor}
                    if($smb1LogicalSwitchVirtualNetworkAdaptor -ne $null){Remove-SCLogicalSwitchVirtualNetworkAdapter -LogicalSwitchVirtualNetworkAdapter $smb1LogicalSwitchVirtualNetworkAdaptor}
                    if($smb1LogicalSwitchVirtualNetworkAdaptor -ne $null){Remove-SCLogicalSwitchVirtualNetworkAdapter -LogicalSwitchVirtualNetworkAdapter $smb2LogicalSwitchVirtualNetworkAdaptor}
                }

                if($StagesOfDevlopment.Contains('11')){
                    Write-Host "Removing SC Uplink Port Profile Set"
                    $UplinkPortProfileSet = Get-SCUplinkPortProfileSet -Name $uplinkPortProfileName -VMMServer $vmm | Remove-SCUplinkPortProfileSet
                }

                if($StagesOfDevlopment.Contains('10')){
                    Write-Host "Removing Sc Logical Switch"
                    $LogicalSwitch = Get-SCLogicalSwitch -Name $switchName -VMMServer $vmm | Remove-SCLogicalSwitch
                }

                if($StagesOfDevlopment.Contains('9')){
                    Write-Host "Removing SC Native Uplink Port Profile"
                    $nativeUplinkPortProfile = Get-SCNativeUplinkPortProfile -Name $uplinkPortProfileName -VMMServer $vmm | Remove-SCNativeUplinkPortProfile
                }

                if($StagesOfDevlopment.Contains('8')){
                    Write-Host "Removing SC Static IP Address Pool"
                    $StaticIpAddressPool = Get-SCStaticIPAddressPool -Name $poolName -VMMServer $vmm | Remove-SCStaticIPAddressPool
                }

                if($StagesOfDevlopment.Contains('7')){
                    #no code to delete the gateway
                }

                if($StagesOfDevlopment.Contains('6')){
                    Write-Host "removing the SCVMSubnet"
                    $mgmtSubnet = Get-SCVMSubnet -Name "$($mgmtName)_0" -VMMServer $vmm | Remove-SCVMSubnet
                    write-host "removed Mgmt SCVM Subnet"
                    $smb1Subnet = Get-SCVMSubnet -Name "$($smb1Name)_0" -VMMServer $vmm | Remove-SCVMSubnet
                    write-host "removed smb1 subnet"
                    $smb2Subnet = Get-SCVMSubnet -Name "$($smb2Name)_0" -VMMServer $vmm | Remove-SCVMSubnet
                    write-host "removed smb2 subnet"  
                }

                if($StagesOfDevlopment.Contains('5')){
                    Write-Host "removing SCVMNetwork"
                   $mgmtnetwork = Get-SCVMNetwork -Name $mgmtName -VMMServer $vmm | Remove-SCVMNetwork
                    Write-Host "removed mgmt"
                   $smb1network = Get-SCVMNetwork -Name $smb1Name -VMMServer $vmm | Remove-SCVMNetwork
                   write-host "removed smb1"
                   
                   $smb2network = Get-SCVMNetwork -Name $smb2Name -VMMServer $vmm | Remove-SCVMNetwork
                   write-host "removed smb2"
                }

                if($StagesOfDevlopment.Contains('4')){
                write-host "removing logical network defination"
                    $mgmtLogicalNetworkdefination = Get-SCLogicalNetworkDefinition -Name "$($mgmtName)_0" -VMMServer $vmm | Remove-SCLogicalNetworkDefinition
                    write-host "removed mgmt"
                    
                    $smb1LogicalNetworkdefination = Get-SCLogicalNetworkDefinition -Name "$($smb1Name)_0" -VMMServer $vmm | Remove-SCLogicalNetworkDefinition
                     write-host "removed smb1"
                    $smb2LogicalNetworkdefination = Get-SCLogicalNetworkDefinition -Name "$($smb2Name)_0" -VMMServer $vmm | Remove-SCLogicalNetworkDefinition
                     write-host "removed smb2"
                }

                if($StagesOfDevlopment.Contains('3')){
                    #No cmdlet to remove SubnetVlan
                }

                if($StagesOfDevlopment.Contains('2')){
                    write-host "removing logical network"
                    $mgmtLogicalNetwork = Get-SCLogicalNetwork -Name $mgmtName -VMMServer $vmm | Remove-SCLogicalNetwork
                    $smb1LogicalNetwork = Get-SCLogicalNetwork -Name $smb1Name -VMMServer $vmm | Remove-SCLogicalNetwork
                    $smb2LogicalNetwork = Get-SCLogicalNetwork -Name $smb2Name -VMMServer $vmm | Remove-SCLogicalNetwork
                }
                

        }catch{
            write-host "Exception occured while deleting the created resources "+$_.Exception.Message
	        $errmsg="Exception Occured"+$_.Exception.Message
	        $returnval=$returnval+ "FAIL"
	        $returnval=$returnval+$errmsg

            return $returnval
        }
    }

	return $returnval
}