﻿param(	[string]$deviceType, 
		[string]$mode,
		[string]$enableOverride)
		
$OMPSInstallKey = "HKLM:\Software\Microsoft\System Center Operations Manager\12\Setup\Powershell\V2"
$regKey = get-item $OMPSInstallKey
$OMPSInstallPath = $regKey.GetValue("InstallDirectory")
$omModule = $OMPSInstallPath + "OperationsManager"
Import-Module -Name $omModule
Import-Module OperationsManager
$fqdn = [System.Net.Dns]::GetHostByName(($env:computerName))
$computerName = $fqdn.HostName
$mg = New-Object -TypeName "Microsoft.EnterpriseManagement.ManagementGroup" -ArgumentList @($computerName)
$datentime = Get-Date

Write-Host "$datentime :: Device Type = $deviceType"
Write-Host "$datentime :: Mode = $mode"
Write-Host "$datentime :: Enable Override = $enableOverride"

function createOverride
{
	param($overrideValue)
	$mps=Get-SCOMManagementPack -DisplayName "Dell Server"
	$classes = Get-SCOMClass -Name "Dell.SDK.Server.System"
	foreach($class in $classes){
		$Monitors = Get-SCOMMonitor -Target $class
		foreach($monitor in $Monitors){
			if(($monitor.Name.Contains("PerformanceUnitMonitor")) -and ($monitor.Enabled -eq "False")){
				Write-Host "$datentime :: Monitor : $monitor"
				$Target= Get-SCOMClass -id $monitor.Target.id
				$overridname=$monitor.name+".Override"
				$override = New-Object Microsoft.EnterpriseManagement.Configuration.ManagementPackMonitorPropertyOverride($overrideMp,$overridname)
				$override.Monitor = $monitor
				$override.Property = 'Enabled'
				$override.Value = $overrideValue
				$override.Context = $Target
				$override.DisplayName = $overridname
				$override.Enforced = 'true'
			}
		}
	}
	$overrideMp.Verify()
	$overrideMp.AcceptChanges()
	$mg.ImportManagementPack($overrideMp)
}

try
{
	if($deviceType -eq "Server")
	{
		$overrideMPPresent = Get-SCOMManagementPack -Name "Dell.PerformanceUnit.Monitor.Override"
		if($overrideMPPresent -eq $null )
		{
			Write-Host "$datentime :: Creating Override MP and Overrides"
			$mpID = "Dell.PerformanceThreshold.Monitor.Override"
			$mg = New-Object Microsoft.EnterpriseManagement.ManagementGroup($computerName)
			$mpStore = New-Object Microsoft.EnterpriseManagement.Configuration.IO.ManagementPackFileStore
			$overrideMp = New-Object Microsoft.EnterpriseManagement.Configuration.ManagementPack($mpID, $mpID, (New-Object Version(1, 0, 0)), $mpStore)
			$overrideMp.DisplayName = "Dell.Performance.Threshold.Monitor.Override"
			$overrideMp.Description = "Dell Server's Override Manangement Pack for Threshold Monitoring"
			Write-Host "$datentime :: Override MP Name : $overrideMp"
		}
		if( $mode -eq "Scalable" )
		{
			createOverride -overrideValue 'False'
			return "SUCCESS"
		}
		elseif( $mode -eq "Detailed" )
		{
			if( $enableOverride -eq "Yes")
			{
				createOverride -overrideValue 'True'
			}
			elseif( $enableOverride -eq "No" )
			{
				createOverride -overrideValue 'False'
			}
			return "SUCCESS"
		}
		else
		{
			return "FAIL"
		}
	}
	else
	{
		return "FAIL"
	}
}
catch [Exception]
{
	return "FAIL"
}

