﻿param($nfspath, $serviceTagFolders)
#$nfspath = "C:\Windows\Temp\Jsonfiles\NetworkSwitch"
#$serviceTagFolders =  @("ABC")
$ErrorActionPreference = "SilentlyContinue"
$OMPSInstallKey = "HKLM:\Software\Microsoft\System Center Operations Manager\12\Setup\Powershell\V2"
$regKey = get-item $OMPSInstallKey
$OMPSInstallPath = $regKey.GetValue("InstallDirectory")
$omModule = $OMPSInstallPath + "OperationsManager"
Import-Module -Name $omModule
$fqdn = [System.Net.Dns]::GetHostByName(($env:computerName))
$computerName = $fqdn.HostName
$mg = New-Object -TypeName "Microsoft.EnterpriseManagement.ManagementGroup" -ArgumentList @($computerName)
$discoveryData = new-Object -TypeName "Microsoft.EnterpriseManagement.ConnectorFramework.IncrementalDiscoveryData"
$Connector = Get-SCOMConnector -Name "Dell Device Monitoring Connector"
if($Connector -eq $null){
    $connectorInst = new-Object -TypeName 'Microsoft.EnterpriseManagement.ConnectorFramework.ConnectorInfo'
    $connectorInst.Description = "Connector for Monitoring Dell Devices"
    $connectorInst.Name = "Dell Device Monitoring Connector"
    $connectorInst.DisplayName = "Powershell Inbound connector";
    $ConnectorGuid = [System.Guid]::NewGuid().Guid
    $cfa = $mg.GetConnectorFrameworkAdministration()
    $Connector = $cfa.Setup($connectorInst, $ConnectorGuid)
}
Function psDebugLog{		
	param($level, $message)
	if (($level -gt 0)){
		$currentTime = Get-Date -f "yyyy-MM-dd_HH.mm.ss"
		Out-File -FilePath $LogFileLocation -InputObject ($currentTime + " :: " + $message) -Append
	}
}
function GetMonitoringClass{
    param($name)
    $getscomClass= Get-SCOMClass -Name $name
    $retClass = $mg.GetMonitoringClass($getscomClass.Id)
    return $retClass
}

function GetMonitoringRelation{
    param($name)
    $getscomRelation= Get-SCOMRelationship -Name $name
    $retRelation = $mg.GetMonitoringRelationshipClass($getscomRelation.Id)
    return $retRelation
}
function ConvertPSObjectToHashtable{
    param (
        [Parameter(ValueFromPipeline)]
        $InputObject
    )

    process
    {
        if ($null -eq $InputObject)
        { 
            return $null
        }

        if ($InputObject -is [System.Collections.IEnumerable] -and $InputObject -isnot [string])
        {
            $collection = @(
                foreach ($object in $InputObject) { ConvertPSObjectToHashtable $object }
            )
            Write-Output -NoEnumerate $collection
        }
        elseif ($InputObject -is [psobject])
        {
            $hash = @{}

            foreach ($property in $InputObject.PSObject.Properties)
            {
                $hash[$property.Name] = ConvertPSObjectToHashtable $property.Value
            }
            $hash
        }
        else
        {
            $InputObject
        }
    }
} 

function cleanUpinstances{
    $deletedInsts = @{}
    $jsonObj = ConvertPSObjectToHashtable $jsonObj
    if($jsonObj._deleted_comps)
    {
        $deletedInsts = $deletedInsts + $jsonObj._deleted_comps
    }
    if($jsonObj._deleted_instances)
    {
        $deletedInsts = $deletedInsts + $jsonObj._deleted_instances
    }
    
    if($deletedInsts)
    {

  
    
        foreach($comp in $deletedInsts.getEnumerator())
        {        
            $classname = "Dell.NetworkSwitch." + $comp.Name         
            $comp
            $objsTobeDel = Get-SCOMClass -Name $className | Get-SCOMClassInstance
            foreach($jsobj in $comp.Value)
            {
                $jkey = $jsobj.Key
                foreach($obj in $objsTobeDel)
                {
                    $instancekey = $obj.GetMonitoringPropertyValue(($obj.GetMonitoringProperties() | where {$_.Name -eq 'key'}))
                    if(($obj.FullName -match $rootKey) -and ($instancekey -eq $jkey))
                    {
                    $obj.FullName
                    $instancekey
                        #Write-Host "DELETING "$obj.FullName -ForegroundColor DarkRed
                        $discoveryData.remove($obj)
                    }
                }
            }
        }
    }
}


function CreateContainmentRelation{
param($relation, $sourceInst, $targetInst)
$containmentRelationship = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringRelationshipObject($relation)
$containmentRelationship.SetSource($sourceInst)
$containmentRelationship.SetTarget($targetInst)
$discoveryData.Add($containmentRelationship)
}
function CallCreateContainment{
param($source, $target, $relation)
if($source.MonitoringClass -eq $null){
    $sourceClass = GetMonitoringClass $source
    $sourceInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($sourceClass) }
else{$sourceInst = $source}
if($target.MonitoringClass -eq $null){
    $targetClass = GetMonitoringClass $target
    $targetInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($targetClass) }
else{$targetInst = $target}
$relation = GetMonitoringRelation $relation
CreateContainmentRelation $relation $sourceInst $targetInst
}

$rootInstance = $null
$computerClassID = GetMonitoringClass "Microsoft.Windows.Computer"
$entityClass =      GetMonitoringClass -Name "System.Entity"

#compt-tree.json traversal start
function CreateRootNode{
    param($rootParent, $comptTreeSysObj, $rootKey)
         $rootParentName = $comptTreeSysObj.Name
         $rootParentClassName = "Dell.NetworkSwitch."+$rootParentName
         $childKey_ParentsKey_Map.Add($rootkey, @{$rootParentClassName=$rootKey} )         		
		 $monitoringClass = GetMonitoringClass $rootParentClassName 
    	 $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
         $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)         
         $discoveryData.Add($rootInstance)  
               
         
         If(($systemGeneration -ne "") -and (($manufacturer.ToLower().Contains("dell ")) -or ($manufacturer.ToLower().Contains("dell")) -or ($manufacturer -eq "Not Available") -or ($manufacturer -eq " "))){
		$generation = $systemGeneration.split(" ")[0].replace("G","")
		if(($generation -gt 11) -and ($systemGeneration -ne "Not Available")){
		 $monitoringClass = GetMonitoringClass $rootParentClassName 
    	        $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
               $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
               $discoveryData.Add($rootInstance)   
		 if(($systemGeneration.split(" ")[1] -eq "Monolithic") -or ($systemGeneration.split(" ")[1] -eq "DCS")){
		        $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monolithicServerClass)
                $serverType = "Monolithic"}
	        elseif(($model.ToLower().Contains("poweredge fc")) -or ($model.ToLower().Contains("poweredge mx"))){
		        $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($sledServerClass)
                $serverType = "FC"}
	        elseif($model.ToLower().Contains("poweredge fm")){
		        $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($sledNodeClass)
                #$rootkey = $jsonObj.System."$("NodeID")"
                $serviceTag = $jsonObj.System."$("ServiceTag")"					
                $serverType = "FM"
                $sledName = "Dell FM Server :" + $serviceTag
                $global:fmServerInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($fmServerClass)
                $fmServerInstance.SetMonitoringPropertyValue($fmServerClass.GetMonitoringProperty("ServiceTag"), $serviceTag)
                $fmServerInstance.SetMonitoringPropertyValue($fmServerClass.GetMonitoringProperty("HostManagementServer"), $targetComputer)
	            $fmServerInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $sledName)
	            $discoveryData.Add($fmServerInstance)}
	        elseif(($systemGeneration.split(" ")[1] -eq "Modular") -or ($chassisServiceTag -eq $serviceTag)){
		        $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($modularServerClass)
                $serverType = "Modular"}
	        else{$serverInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monolithicServerClass)} 
            $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
            $discoveryData.Add($rootInstance)}
        else{psDebugLog -level 1 -message "Server Generation Unsupported (<11) "
	    }
    }
    else{psDebugLog -level 1 -message "ServerGeneration is empty - Manufacturer is not Dell"}     
         
        
    if($rootInstance -ne $null){    
   
        if($targetComputer -ne $computerName){
            $healthServiceClassID = GetMonitoringClass "Microsoft.SystemCenter.HealthService"
            $healthServiceShudManageEntityClassID = GetMonitoringRelation "Microsoft.SystemCenter.HealthServiceShouldManageEntity"
            $healthServiceInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($healthServiceClassID) 
            $healthServiceInstance.SetMonitoringPropertyValue($computerClassID.GetMonitoringProperty("PrincipalName"),$targetComputer)
            
            $discoveryData.Add($healthServiceInstance)


            CreateContainmentRelation $healthServiceShudManageEntityClassID $healthServiceInstance $rootInstance}       
            CreateSubsystemNodes $rootParent $rootkey
        #inventory creation start
        if($jsonObj -ne $null){
           CreateInventory $jsonObj
        }
    }
}

function CreateSubsystemNodes{
    param($rootParent,$rootParentName)
    foreach($subSystem in $rootParent.PsObject.Properties){   
        if((!($skipComponents_array -contains $subSystem.Name))){
            if($skipSubsystem_array.Contains($rootParentName)){$subSystemClassName = "Dell.NetworkSwitch." +$rootParentName} 
            else{$subSystemClassName = "Dell.NetworkSwitch."+$subSystem.Name +".Subsystem"
            }      
            if(!($childKey_ParentsKey_Map.Contains($subSystem.Name))){$childKey_ParentsKey_Map.Add($subSystem.Name,@($childKey_ParentsKey_Map.Item($rootParentName)))}
            else{$childKey_ParentsKey_Map.Item($subSystem.Name) = $childKey_ParentsKey_Map.Item($subSystem.Name) + @($childKey_ParentsKey_Map.Item($rootParentName))}   
       
            if(!$skipSubsystem_array.Contains($subSystem.Name)){
                $monitoringClass = GetMonitoringClass $subSystemClassName              
                $classInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)              
                $parentkeymap = $childKey_ParentsKey_Map.Item($subSystem.Name)               
                foreach($parent in $parentkeymap.keys){
                    if(!$skipSubsystem_array.Contains($parentkeymap.$parent)){
                        $parentClass = GetMonitoringClass $parent  
                        if($parent -eq "Microsoft.Windows.Computer"){
                            $classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty("PrincipalName"),$parentkeymap.$parent)  
                        }
                        else{$classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty("Key"),$parentkeymap.$parent)
                        }
                    }
                }
                if($monitoringClass.PropertyCollection.Name.Contains("Component")){
                     $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Component"),"SubSystem")
                }
               			
				$classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$subSystem.Name)           
                $classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $subSystem.Name)
                
                 $discoveryData.Add($classInstance)               
            }
           
            if($subSystem.Value -is [system.array]){        
               foreach($leaf in $subSystem.Value){    
                   $leafNodeClassName = "Dell.NetworkSwitch."+$subSystem.Name                   
                   $childKey_ParentsKey_Map.Add($leaf,@(@{$subSystemClassName = $subSystem.Name})+ $childKey_ParentsKey_Map.Item($subSystem.Name))
               }$childKey_ParentsKey_Map.Remove($subSystem.Name)
            }
            else{
                foreach($value in $subSystem.value.psObject.Properties.name){
                   if(!($childKey_ParentsKey_Map.Contains($value))){
                        if($skipSubsystem_array.Contains($rootParentName)){$subSystemClassName = "Dell.NetworkSwitch." +$rootParentName} 
                        else{$subSystemClassName = "Dell.NetworkSwitch."+$subSystem.Name +".Subsystem"}                
                       
                        $childKey_ParentsKey_Map.Add($value, @(@{$subSystemClassName = $subSystem.Name}))
                        }
               }
               CreateSubsystemNodes $subSystem.value $subSystem.Name
               if($skipSubsystem_array.Contains($value)){$childKey_ParentsKey_Map.Remove($value)}
            }
        }
        if(($skipSubsystem_array.Contains($subsystem.Name))){
            $childKey_ParentsKey_Map.Remove($subsystem.Name)}
    }
}
function CreateInventory()
{    
    param($jsonObj, $notAvailable) 
    
    #$ipAddress = $jsonObj.System.ManagementIP
    #$parts = $ipAddress.Split("//")
     #$RemoteAccessIP = $parts[2] 
     $RemoteAccessIP =  $jsonObj.System.SwitchIPv4
      
     foreach($Component in $jsonObj.PsObject.Properties) {
            if(!($skipComponents_array -contains $Component.Name)){          
                $cmpclass = "Dell.NetworkSwitch." + $Component.Name                         
                psDebugLog -level 1 -message  ("Inventory started for component - " + $Component.Name)
			    foreach($instance in $Component.Value) {
                    $monitoringClass = GetMonitoringClass $cmpclass                    
                    $classInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
                    $parentkeymap = $childKey_ParentsKey_Map.Item($instance.key)              
                    foreach($parent in $parentkeymap.keys){
                        if(!$skipSubsystem_array.Contains($parentkeymap.$parent)){
                            $parentClass = GetMonitoringClass $parent                              
                            $classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty("Key"),$parentkeymap.$parent)
                        }
                    }
                    foreach($attribute in $instance.PsObject.Properties) {
                   
                        if($monitoringClass.PropertyCollection.Name.Contains($attribute.Name)){	                                      
                        $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($attribute.Name), $attribute.Value)
					}
                    }
                    if($monitoringClass.PropertyCollection.Name.Contains("Component")){
                         $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Component"),$Component.Name)
                    }                    
                    if($Component.Name -eq "System"){
                    $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("ManagementIPv4Address"), $RemoteAccessIP)
                    $classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $sysDisplayName)                    
                    }                  
                    else{$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.Key)
                   
                   }
                    $discoveryData.Add($classInstance)
                    psDebugLog -level 1 -message  ($instance.Key + " instance discovered")
                }
                psDebugLog -level 1 -message  ("Inventory completed for component - " + $Component.Name)
		    }
          #  else{
            #    if($Component.Name -eq "_deleted_comps"){ 
             #       foreach($modifiedComp in $Component.Value.PsObject.Properties) {
                   
           #             if($modifiedComp.Name -eq "BIOS"){Get-SCOMClassInstance -DisplayName $rootKey| ForEach-Object {if($_.FullName.Contains($rootkey)){$inst = $_}}}
            #                                         else{Get-SCOMClassInstance -DisplayName $modifiedComp.Name | ForEach-Object {if($_.FullName.Contains($rootkey)){$inst = $_}}}
            #            if($inst -ne $null){$discoveryData.Remove($inst)}
           #         }
           #     } 
           #     elseif($Component.Name -eq "_deleted_instances"){  
            #        foreach($modifiedComp in $Component.Value.PsObject.Properties) {
            #            foreach($instance in $modifiedComp.Value) { 
            #            	$inst = $null
                            
            #                if($modifiedComp.Name -eq "License"){Get-SCOMClassInstance -DisplayName $instance.LicenseDescription | ForEach-Object {if($_.FullName.Contains($rootkey)){$inst = $_}}}
            #                elseif($modifiedComp.Name -eq "HostNIC"){Get-SCOMClassInstance -DisplayName $instance.Name | ForEach-Object {if($_.FullName.Contains($rootkey)){$inst = $_}}}
            ##                else{Get-SCOMClassInstance -DisplayName $instance.Key | ForEach-Object {if($_.FullName.Contains($rootkey)){$inst = $_}}}
            #                if($inst -ne $null){$discoveryData.Remove($inst)}
           #             }
           #         }                
            #    }
           #}
     }
}
function WriteToAppliance{
    param($Device, $passed, $failed)
    $creds = @{"scomdevices"=@{"PassedDevices"= $passed} + @{"FailedDevices"= $failed}}
	$jsonDeviceObj = $creds | ConvertTo-Json
    return $jsonDeviceObj
}
$jsonObj = $null
#$serviceTagFolders = Get-ChildItem $nfspath
$passed = @()
$failed = @()
function CreateLogFile{
param($scriptname, $logDirectory)
try{
	    $ErrorActionPreference = "SilentlyContinue"
	    $TempFolder = $env:SystemRoot + "\Temp"
	    $LogLocation = $TempFolder + "\" + $logDirectory + "\"
	    If(!(Test-Path -path($TempFolder))){
		    # create the directory if not present
		    New-Item $TempFolder -type directory 
	    }
	    If(!(Test-Path -path($LogLocation))){
		    # create the directory if not present
		    New-Item $LogLocation -type directory 
	    }
	    $Global:LogFileLocation = $LogLocation + $scriptname + ".log"
	    If(!(Test-Path -path($LogFileLocation))){
		    # create the file if it does not exist
		    New-Item $LogFileLocation -type file
	    }
	    Else{
		    $logFileSize = Get-ChildItem $LogFileLocation | ForEach-Object {($_.Length/1KB)}
		    If ($logFileSize -gt 512){
			    # existingLogFile is greater than 512 KB
			    $archiveTime = Get-Date -f "yyyy-MM-dd_HH.mm" 
			    Rename-Item $LogFileLocation ("ArchivedLog_" + $scriptname + "_" + $archiveTime + ".log")
			    New-Item $LogFileLocation ChildItem-type file
		    }                    
	    }
    }
    catch{
	    $ErrorMessage = $_.Exception.Message
	    $FailedItem = $_.Exception.ItemName
	    $message = "Exception in creating log file. Exception-Message:" + $ErrorMessage + $FailedItem
	    $api.LogScriptEvent("NetworkSwitchDiscovery.ps1",1345,0,$message)
    }

}
$logLevel = 1
#$discOverrides = Get-SCOMDiscovery -Name "Dell.Server.Discovery" | Get-SCOMOverride -ErrorAction SilentlyContinue
#if($discOverrides -ne $null){
#    foreach ($override in $discOverrides){
#        if($override.Parameter -eq "LogLevel"){
#            $logLevel = $override.Value}
#    }
#}
foreach($serviceTag in $serviceTagFolders){
    $scriptname = $serviceTag + "_DellNetworkSwitchesDiscovery"
    $logDirectory = "DellNetworkSwitchesDiscovery_log"	
    if($logLevel -eq 1){		
        CreateLogFile $scriptname $logDirectory }   
    try{
        $jsonpath = $nfspath+"\"+$serviceTag+"\Inventory.json"
        if ((Get-Item $jsonpath -ErrorAction Ignore) -ne $null) {
	        $jsonObj = ConvertFrom-Json -InputObject (Get-Content $jsonpath -Raw)
        }

        $comptTreeFilePath = $nfspath+"\"+$serviceTag+"\CompTree.json"
        if ((Get-Item $comptTreeFilePath -ErrorAction Ignore) -ne $null) {
	        $comptTreeObj = ConvertFrom-Json -InputObject (Get-Content $comptTreeFilePath -Raw)
        }
    }
    catch{
        $ErrorMessage = $_.Exception.Message
        $FailedItem = $_.InvocationInfo.ScriptLineNumber
        psDebugLog -level 1 -message  ("Exception occurred : Line - " + $FailedItem + " - " + $ErrorMessage )
        $failed += $serviceTag
        continue
    }
    $childKey_ParentsKey_Map = @{}
    $sysDisplayName = $null
    $rootParent = $comptTreeObj.System

    $comptTreeObj.PsOBject.Properties | ForEach-Object {if($_.Name -eq "System"){$comptTreeSysObj = $_}}
    $skipSubsystem_array = @("Controller", "Enclosure", "iDRAC", "iDRACNIC", "EnclosureEMM", "BIOS", "DirectAttachedPhysicalDisk")
    $skipComponents_array = @("Subsystem" ,"Flash","SysIf", "Processor","StackUnit","SysCores","CpuUtil","PowerSupplyTray","FanTray","SwModule","Chassis", "_deleted_comps", "_deleted_instances", "_deleted_attribs" , "SystemBoardMetrics" , "SystemMetrics", "FC", "LogicalSystem", "PresenceAndStatusSensor")
    $rootKey = $serviceTag
    $SwitchTypes = New-Object System.Collections.ArrayList
    $SwitchTypes.Add("NSeries")
    $SwitchTypes.Add("MSeries")
    $SwitchTypes.Add("SSeries")
    $SwitchTypes.Add("ZSeries")
#    $SwitchTypes.Add("F10")
    $SwitchType = $jsonObj.System."$("SwitchType")"
    $hostName = $jsonObj.System."$("HostName")"
    $model = $jsonObj.System."$("Model")"    
    $targetComputer = $jsonObj.System."$("MachineName")"
    if($targetComputer -eq "Not Available"){$targetComputer = $computerName}

    if(($hostName -ne $null) -and ($hostName -ne " ") -and ($hostName -ne "(none)")){
        $sysDisplayName = $hostName
    }    
    if($SwitchTypes.Contains($SwitchType)){
    cleanUpInstances
        CreateRootNode $rootParent $comptTreeSysObj $rootKey		
        $passed += $serviceTag
    }
    else{$failed += $serviceTag}
    #compt-tree.json traversal end

    $discoveryData.Overwrite($Connector)	
}
$devicesPassedFailed = WriteToAppliance $Device $passed $failed
return $devicesPassedFailed

