param($VMMServer,$DomainUsername,$DelPwd,$tejasFQDN,$otherInfo)
## This script gets the logical network details from scvmm
$debug1 = 1
#Import-Module -Name "virtualmachinemanager"

# Return Value
$returnval = @()
$temp = @("null") 
  
#$DelUserName='tejas\administrator'
#$DelPwd='Dell_123$'
#$VMMServer='scvmm.tejas.com'
#$libShare = '\\orchestrator.tejas.com\SCVMMLib'
#$uuid = '4c4c4544-0039-4810-8051-b2c04f383253'
#$hostProfileName = @("test")
# $newHostProfilename = $hostProfileName + '_' + $uuid



# Create obj
Function Createcredobj($username, $passwd)
{

    try
    {
		$secpasswd = ConvertTo-SecureString  $passwd -AsPlainText -Force
        $mycredsobj = New-Object System.Management.Automation.PSCredential ($username, $secpasswd)
        #$Error
    }
    catch
    {
        $mycredsobj = $null
    }

    return $mycredsobj

}

# Create SCVMM Cred obj    
	
	$otherInfo = $otherInfo -replace "\\n\\r", " "
	$libShare=$otherInfo.split(',')[0]
	$uuid=$otherInfo.split(',')[1]
	$hostProfileName=$otherInfo.split(',')[2]
	$newHostProfilename=$otherInfo.split(',')[3]
	$drivertag = $uuid
	
    $scvmmcreds = Createcredobj $DomainUsername $DelPwd
    if ( $scvmmcreds -eq $null)
    {
        $returnval = $returnval + "FAIL"
		$returnval = $returnval + "Could not create credentail object for the given SCVMM user"
        return $returnval
    }

# A user can have mutiple role, in Tejasconext we are connecting always as DelegatedAdmin Role.

    try 
    {
    	#import SCVMM related cmdlets
		import-module virtualmachinemanager
        $VMMobj = Get-SCVMMServer -ComputerName $VMMServer -Credential $scvmmcreds 
    }
    catch [exception]
    {
        $returnval = $returnval + "FAIL"
		$returnval = $returnval + $Error[0].Exception
        if ($debug1 -eq 0)
		{
			write-host $returnval
		}
		return $returnval
    }

    if ( $VMMobj -eq $null)
    {
        $returnval = $returnval + "FAIL"
        $returnval = $returnval + "Could not connect to SCVMM, host not reachable"
        if ($debug1 -eq 0)
		{
			write-host $returnval
		}
		return $returnval
       
    }
	else
    {
		
		try
		{
			# get library server
            $ErrorActionPreference = "Stop";
            
            $driverpacks=(Get-SCDriverPackage -VMMServer $VMMobj | where {$_.SharePath.ToLower().StartsWith($libShare.ToLower()) -and $_.State -eq "Normal" }) 


		    # No drivers found
			if ($driverpacks -eq $null -or $driverpacks.Count -eq 0)
			{
				if ($debug1 -eq 0)
				{
					write-host "No drivers found in library share for the given server $uuid"
				}
				$returnval = $returnval + "FAIL"
				$returnval = $returnval + "No drivers found in library share for the given server $uuid"
				
				return $returnval
				
			}
			else # Found some drivers
			{
				if ($debug1 -eq 0)
				{
					write-host "Drivers found for the given path in SCVMM"
				}
               
               
                # Set tags for all drivers
                try
                {
                
                    foreach ($driver in $driverpacks )
                    {
                        Set-SCDriverPackage -DriverPackage $driver  -Tag $drivertag -VMMServer $VMMobj | out-null
                    }
                }
                catch [exception]
		        {
					$returnval = $returnval + "FAIL"
				    $returnval = $returnval + "Exception while setting tags for the driver $driver for server $uuid"
				    
				

				    if ($debug1 -eq 0)
				    {
					    write-host $returnval
				    }
				    return $returnval
		        }
                
                # Get the host profile   
                $hostProfile = (Get-ScVMHostProfile -Name $hostProfileName -VMMServer $VMMobj)
                     
                if ( $hostProfile -eq $null )
		        {
				    $returnval = $returnval + "FAIL"
				    $returnval = $returnval + "Host profile not found in SCVMM for $hostProfileName"
					

				    if ($debug1 -eq 0)
				    {
					    write-host $returnval
				    }
				    return $returnval
		        }

                
                # clone the existing host profile
                
				$NewHostProfile = New-SCVMHostProfile -Name $newHostProfilename -VMHostProfile $hostProfile -VMMServer $VMMobj

                if ( $NewHostProfile -eq $null )
		        {
				    $returnval = $returnval + "FAIL"
				    $returnval = $returnval + "Cloning Host profile $hostProfileName failed"
					

				    if ($debug1 -eq 0)
				    {
					    write-host $returnval
				    }
				    return $returnval
		        }

               
                # check if host profile already has tags if yes add new , else append the new
                if ( $NewHostProfile.DriverMatchingTags -eq $null )
                {
                    if ($debug1 -eq 0)
				    {
                        Write-host "No prior tags exists, adding the tag to host profile $newHostProfilename" 
                    }

                    try { [void] (Set-SCVMHostProfile -VMHostProfile $NewHostProfile -DriverMatchingTag $drivertag -VMMServer $VMMobj | Out-Null) }
                    catch [exception]
                    {
                       $returnval = $returnval + "FAIL"
			           $returnval = $returnval + "Exception occured while adding the tag drivertag to host profile $newHostProfilename"
				      
				

				        if ($debug1 -eq 0)
				        {
					        write-host $returnval
				        }
				        return $returnval
                    }
                }
                else
                {
                    if ($debug1 -eq 0)
				    {
                        Write-host "Already prior tags exists, Appending the tag to host profile $newHostProfilename" 
                    }

                    try {
                    $tagList = $NewHostProfile.DriverMatchingTags
                    $tagList.Add($drivertag)
                    [void] (Set-SCVMHostProfile -VMHostProfile $NewHostProfile -DriverMatchingTag $tagList -VMMServer $VMMobj | Out-Null)}
                    catch [exception]
                    {
                       $returnval = $returnval + "FAIL"
			           $returnval = $returnval + "Exception occured while adding the tag drivertag to host profile $newHostProfilename"+$_.Exception.Message
				       
				

				        if ($debug1 -eq 0)
				        {
					        write-host $returnval
				        }
				        return $returnval
                    }
                }
                
				
                $returnval = $returnval + "SUCCESS"
				$returnval = $returnval + "Tagging drivers and updating host profile completed successfully"
				

				if ($debug1 -eq 0)
				{
					write-host $returnval
				}
				return $returnval
			}

		}
		catch [exception]
		{
				$returnval = $returnval + "FAIL"
			    $returnval = $returnval + "Exception occured tagging drivers for server $uuid"+$_.Exception.Message
				
				

				if ($debug1 -eq 0)
				{
					write-host $returnval
				}
				return $returnval
		}

}