﻿param($nfspath, $serviceTagFolders)
#$nfspath = "C:\nfsshare\Server"
#$serviceTagFolders = Get-ChildItem $nfspath | Foreach-Object {$_.Name}
$ErrorActionPreference = "SilentlyContinue"
$OMPSInstallKey = "HKLM:\Software\Microsoft\System Center Operations Manager\12\Setup\Powershell\V2"
$regKey = get-item $OMPSInstallKey
$OMPSInstallPath = $regKey.GetValue("InstallDirectory")
$omModule = $OMPSInstallPath + "OperationsManager"
Import-Module -Name $omModule
$fqdn = [System.Net.Dns]::GetHostByName(($env:computerName))
$computerName = $fqdn.HostName
$mg = New-Object -TypeName "Microsoft.EnterpriseManagement.ManagementGroup" -ArgumentList @($computerName)
$discoveryData = new-Object -TypeName "Microsoft.EnterpriseManagement.ConnectorFramework.IncrementalDiscoveryData"
$Connector = Get-SCOMConnector -Name "Dell Device Monitoring Connector"
$logLevel = 0

$script:storageTreeIditer = $null
$script:treespacing = 0
$storageTreeMap =@{}

$jsonObj = $null
$rootKey = $null
$sysDisplayName = $null

$passed = @()
$failed = @()

$rootInstance = $null

$skipSubsystem_array = @("Controller", "Enclosure", "iDRAC", "iDRACNIC", "EnclosureEMM", "BIOS", "DirectAttachedPhysicalDisk","ControllerSensor", "EnclosureSensor", "PresenceAndStatusSensor")
$skipComponents_array = @("Subsystem" , "_deleted_comps", "_deleted_instances", "_deleted_attribs" , "PCIDevice", "Video", "Fan", "SystemBoardMetrics" , "SystemMetrics", "FC", "LogicalSystem")
$Component_array = @("System","VFlash", "iDRACNIC","Sensors", "Storage", "Controller", "ControllerSensor", "ControllerBattery", "Enclosure", "EnclosureSensor", "PhysicalDisk", "VirtualDisk", "EnclosureEMM", "Memory", "CPU", "iDRAC", "FC", "NIC", "PCIDevice", "Fan", "PowerSupply", "Video", "License", "HostNIC", "BIOS", "Sensors_Amperage", "Sensors_Temperature", "Sensors_Voltage", "Sensors_Intrusion", "Sensors_Battery", "Sensors_Fan", "PresenceAndStatusSensor", "EnclosureEMM", "EnclosurePSU", "PhysicalDisk", "EnclosureSensor", "PCIeSSDExtender", "PCIeSSDExtender", "PCIeSSDBackPlane","PCIeSSDBackPlane","PCIeSSDDisk", "EnclosureSensor", "EnclosureFanSensor", "EnclosureTempSensor")
	
if($Connector -eq $null){
    $connectorInst = new-Object -TypeName 'Microsoft.EnterpriseManagement.ConnectorFramework.ConnectorInfo'
    $connectorInst.Description = "Connector for Monitoring Dell Devices"
    $connectorInst.Name = "Dell Device Monitoring Connector"
    $connectorInst.DisplayName = "Powershell Inbound connector";

    $ConnectorGuid = [System.Guid]::NewGuid().Guid
    $cfa = $mg.GetConnectorFrameworkAdministration()
    $Connector = $cfa.Setup($connectorInst, $ConnectorGuid)
}

function ConvertPSObjectToHashtable{
    param (
        [Parameter(ValueFromPipeline)]
        $InputObject
    )

    process
    {
        if ($null -eq $InputObject)
        { 
            return $null
        }

        if ($InputObject -is [System.Collections.IEnumerable] -and $InputObject -isnot [string])
        {
            $collection = @(
                foreach ($object in $InputObject) { ConvertPSObjectToHashtable $object }
            )
            Write-Output -NoEnumerate $collection
        }
        elseif ($InputObject -is [psobject])
        {
            $hash = @{}

            foreach ($property in $InputObject.PSObject.Properties)
            {
                $hash[$property.Name] = ConvertPSObjectToHashtable $property.Value
            }
            $hash
        }
        else
        {
            $InputObject
        }
    }
} 

function psDebugLog{		
	param($level, $message)
	if (($level -gt 0)){
		$currentTime = Get-Date -f "yyyy-MM-dd_HH.mm.ss"
		Out-File -FilePath $LogFileLocation -InputObject ($currentTime + " :: " + $message) -Append
	}
}

function GetMonitoringClass{
    param($name)
    $getscomClass= Get-SCOMClass -Name $name
    $retClass = $mg.GetMonitoringClass($getscomClass.Id)
    return $retClass
}

function GetMonitoringRelation{
    param($name)
    $getscomRelation= Get-SCOMRelationship -Name $name
    $retRelation = $mg.GetMonitoringRelationshipClass($getscomRelation.Id)
    return $retRelation
}

function CreateContainmentRelation{
    param($relation, $sourceInst, $targetInst)
    $containmentRelationship = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringRelationshipObject($relation)
    $containmentRelationship.SetSource($sourceInst)
    $containmentRelationship.SetTarget($targetInst)
    $discoveryData.Add($containmentRelationship)
}

function CallCreateContainment{
    param($source, $target, $relation)
    if($source.MonitoringClass -eq $null){
        $sourceClass = GetMonitoringClass $source
        $sourceInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($sourceClass) }
    else{$sourceInst = $source}
    if($target.MonitoringClass -eq $null){
        $targetClass = GetMonitoringClass $target
        $targetInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($targetClass) }
    else{$targetInst = $target}
    $relation = GetMonitoringRelation $relation
    CreateContainmentRelation $relation $sourceInst $targetInst
}

function storageMap
{
    param($ParentKeyId,$subSystemClassName, $MPKeyName, $MPKeyValue, $classInstance)                
    $rkey = $ParentKeyId 
    #write-host "ENTER ADPST "$MPKeyName" "$MPKEYVALUE -ForegroundColor Magenta
    while($storageTreeMap.Item($rkey) -ne $null)
    {
        [Array]$parentArray = $storageTreeMap.Item($rkey)
        #write-host "Adding hosting Parent KEY NAME $($parentArray[2]) KEY $($parentArray[3])"
        $parentClass = GetMonitoringClass $parentArray[1]
        $classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty($parentArray[2]), $parentArray[3])
        [int]$rkey = $parentArray[0]
    }
	[Array]$tmpArray = @($ParentKeyId, $subSystemClassName,$MPKeyName, $MPKeyValue)
    $script:storageTreeIditer = $script:storageTreeIditer + 1
    $storageTreeMap.Add($script:storageTreeIditer, [Array]$tmpArray)
  
    return $script:storageTreeIditer 
}

$computerClassID = GetMonitoringClass "Microsoft.Windows.Computer"
$entityClass = GetMonitoringClass -Name "System.Entity"
$monolithicServerClass = GetMonitoringClass "Dell.Rack.Servers"
$sledServerClass = GetMonitoringClass "Dell.SDK.SledServer"
$sledNodeClass = GetMonitoringClass "Dell.SDK.SledNode"
$modularServerClass = GetMonitoringClass "Dell.Modular.Servers"
$fmServerClass = GetMonitoringClass "Dell.SDK.FMServer"

$hash_ServerGroupMap = @{"Dell.Base.HardwareGroup.Contains.Dell.Servers.Group" = @("Dell.Base.HardwareGroup", "Dell.Servers.Group");
                        "Dell.Servers.Group.Contains.Dell.Rack.Servers.Group" = @("Dell.Servers.Group", "Dell.Rack.Servers.Group");
                        "Dell.Servers.Group.Contains.Dell.Modular.Servers.Group" = @("Dell.Servers.Group", "Dell.Modular.Servers.Group");
                        "Dell.Servers.Group.Contains.Dell.SDK.SledGroup"= @("Dell.Servers.Group","Dell.SDK.SledGroup")}

#CreateContainment relation with Dell Hardware Group
foreach ($key in $hash_ServerGroupMap.keys){
    CallCreateContainment $hash_ServerGroupMap.Item($key)[0] $hash_ServerGroupMap.Item($key)[1] $key
}

function cleanUpinstances{
    $deletedInsts = @{}
    if($jsonObj._deleted_comps)
    {
        $deletedInsts = $deletedInsts + $jsonObj._deleted_comps
    }
    if($jsonObj._deleted_instances)
    {
        $deletedInsts = $deletedInsts + $jsonObj._deleted_instances
    }
    
    if($deletedInsts)
    {
        foreach($comp in $deletedInsts.getEnumerator())
        {
            $classname = "Dell.SDK.Server."+$comp.Name
            $objsTobeDel = Get-SCOMClass -Name $className | Get-SCOMClassInstance
            foreach($jsobj in $comp.Value)
            {
                $jkey = $jsobj.Key
                foreach($obj in $objsTobeDel)
                {
                    $instancekey = $obj.GetMonitoringPropertyValue(($obj.GetMonitoringProperties() | where {$_.Name -eq ‘key’}))
                    if(($obj.FullName -match $rootKey) -and ($instancekey -eq $jkey))
                    {
                        #Write-Host "DELETING "$obj.FullName -ForegroundColor DarkRed
                        $discoveryData.remove($obj)
                    }
                }
            }
        }
    }
}

function cleanUpSubsystems{
    $deletedInsts = @{}
    if($jsonObj._deleted_comps)
    {
        $deletedInsts = $jsonObj._deleted_comps
    }

    if($deletedInsts)
    {
        foreach($comp in $deletedInsts.getEnumerator())
        {
            $classname = "Dell.SDK.Server."+$comp.Name+".Subsystem"
            $objsTobeDel = Get-SCOMClass -Name $className | Get-SCOMClassInstance

            $jkey = $comp.Name
            foreach($obj in $objsTobeDel)
            {
                $instancekey = $obj.GetMonitoringPropertyValue(($obj.GetMonitoringProperties() | where {$_.Name -eq ‘key’}))
                if(($obj.FullName -match $rootKey) -and ($instancekey -eq $jkey))
                {
                    #Write-Host "SUBSYSTEM DELETING "$obj.FullName -ForegroundColor DarkRed
                    $discoveryData.remove($obj)
                }
            }
            
        }
    }
}

#compt-tree.json traversal start
function CreateRootNode{
    param($rootParent, $rootParentName, $rootKey)
    $rootParentClassName = "Dell.SDK.Server."+$rootParentName
	$monitoringClass = GetMonitoringClass $rootParentClassName 

    if(($systemGeneration -ne "") -and (($manufacturer.ToLower().Contains("dell ")) -or ($manufacturer.ToLower().Contains("dell")) -or ($manufacturer -eq "Not Available") -or ($manufacturer -eq " ")))
	{
		$generation = $systemGeneration.split(" ")[0].replace("G","")
		if(($generation -gt 11) -and ($systemGeneration -ne "Not Available"))
		{
		    $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
		    $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
		    $discoveryData.Add($rootInstance)   
			if(($systemGeneration.split(" ")[1] -eq "Monolithic") -or ($systemGeneration.split(" ")[1] -eq "DCS"))
			{
				$rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monolithicServerClass)
				$serverType = "Monolithic"
			}
			elseif(($model.ToLower().Contains("poweredge fc")) -or ($model.ToLower().Contains("poweredge mx")))
			{
				$rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($sledServerClass)
				$serverType = "FC"
			}
	        elseif($model.ToLower().Contains("poweredge fm"))
			{
		        $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($sledNodeClass)
                #$rootkey = $jsonObj.System."$("NodeID")"
                $serviceTag = $jsonObj.System."$("ServiceTag")"					
                $serverType = "FM"
                $sledName = "Dell FM Server :" + $serviceTag
                $global:fmServerInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($fmServerClass)
                $fmServerInstance.SetMonitoringPropertyValue($fmServerClass.GetMonitoringProperty("ServiceTag"), $serviceTag)
                $fmServerInstance.SetMonitoringPropertyValue($fmServerClass.GetMonitoringProperty("HostManagementServer"), $targetComputer)
	            $fmServerInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $sledName)
	            $discoveryData.Add($fmServerInstance)
			}
	        elseif(($systemGeneration.split(" ")[1] -eq "Modular") -or ($chassisServiceTag -eq $serviceTag))
			{
		        $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($modularServerClass)
                $serverType = "Modular"
			}
	        else
			{
				$serverInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monolithicServerClass)
			} 
			$rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
			$discoveryData.Add($rootInstance)}
        else
		{
			psDebugLog -level 1 -message "Server Generation Unsupported (<11) "
	    }
    }
    else
	{
		psDebugLog -level 1 -message "ServerGeneration is empty - Manufacturer is not Dell"
	} 
   
    if($rootInstance -ne $null)
	{
        $hash_ServerTypesMap = @{"Monolithic" = @("Dell.Rack.Servers.Group.Contains.Dell.Rack.Servers","Dell.Rack.Servers.Group", $rootInstance);
                            "Modular" = @("Dell.Modular.Servers.Group.Contains.Dell.Modular.Servers","Dell.Modular.Servers.Group", $rootInstance);
							"FC" = @("Dell.SDK.SledGroup.Contains.Dell.SDK.Sled","Dell.SDK.SledGroup", $rootInstance);
							"FM" = @("Dell.SDK.SledGroup.Contains.Dell.SDK.FMServer", "Dell.SDK.SledGroup", $fmServerInstance)}    
		CallCreateContainment $hash_ServerTypesMap.Item($serverType)[1] $hash_ServerTypesMap.Item($serverType)[2] $hash_ServerTypesMap.Item($serverType)[0]
		
		if($targetComputer -ne $computerName)
		{
			$healthServiceClassID = GetMonitoringClass "Microsoft.SystemCenter.HealthService"
			$healthServiceShudManageEntityClassID = GetMonitoringRelation "Microsoft.SystemCenter.HealthServiceShouldManageEntity"
			$healthServiceInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($healthServiceClassID) 
			$healthServiceInstance.SetMonitoringPropertyValue($computerClassID.GetMonitoringProperty("PrincipalName"),$targetComputer)
			$discoveryData.Add($healthServiceInstance)
			CreateContainmentRelation $healthServiceShudManageEntityClassID $healthServiceInstance $rootInstance
		}
		if($serverType -eq "FM"){
			CreateContainmentRelation $healthServiceShudManageEntityClassID $healthServiceInstance $fmServerInstance
			CallCreateContainment $fmServerInstance $rootInstance "Dell.SDK.FMServer.Contains.Dell.SDK.SledNode"
		}

		$keyid = storageMap -ParentKeyId 0 -subSystemClassName "Microsoft.Windows.Computer" -MPKeyName "PrincipalName" -MPKeyValue $targetComputer -classInstance $rootInstance
		#Here 0 is passed as computer is not hosted
		$keyid = storageMap -ParentKeyId 0 -subSystemClassName $rootParentClassName -MPKeyName "Key" -MPKeyValue $rootKey -classInstance $rootInstance
		populateInventory -componentName $rootParentName  -MPKeyName "Key" -MPKeyValue $rootKey -classInstance $rootInstance -monitoringClass $monitoringClass
		CreateSubsystemNodes $rootParent "Key" $rootParentClassName $rootkey 0 $keyid
    }
}

function CreateSubsystemNodes{
    param($subTree, $rootKeyName, $mpKey,$rootParentClassName, $treespacing, $ParentKeyId)
	
    $subsystemparent = $ParentKeyId
    $treespacing = $treespacing+1    
    $classname = $rootParentClassName
    $parentclassname = $rootParentClassName
    $parentkey = $mpKey
    foreach($subSystem in $subTree.getEnumerator())
    {   
        $subsystemparent = $ParentKeyId         
        if(!($skipComponents_array -contains $subSystem.Name))
        {
			if($Component_array -contains $subSystem.Name)
			{
				if($skipSubsystem_array -contains $subSystem.Name)
				{
					$classname = "Dell.SDK.Server."+$subSystem.Name
					$leaf = $subSystem.Value
				}
				else
				{
                    if(($rootParentClassName -eq 'Dell.SDK.Server.Controller') -and ($subSystem.Name -eq 'PhysicalDisk'))
                    {
                        $classname = "Dell.SDK.Server.DirectAttachedPhysicalDisk"
                    }
                    else
                    {
                        $classname = "Dell.SDK.Server."+$subSystem.Name+".Subsystem"
                        $rootKeyName = "Key"
                        $leaf = $subSystem.Name
                        $pkey = $ParentKeyId
                       
                        $monitoringClass = GetMonitoringClass $classname 
                        $classInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
                        $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($rootKeyName),$leaf)
                        if($subsystem.Name -eq 'Storage')
                        {
                            $pkey = 1 #System is added after Microsoft.Windows.Computer
                            $leaf = $rootKey
                            $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootKey)
                            $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("DisplayName"),$subsystem.Name)
                        }
                        if($monitoringClass.PropertyCollection.Name.Contains("Component"))
                        {
                            $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Component"),"SubSystem")
                        }
                        $subsystemparent = storageMap -ParentKeyId $pkey -subSystemClassName $classname -MPKeyName $rootKeyName -MPKeyValue $leaf -classInstance $classInstance
                        #Write-Host "Adding Class ",$classname," Inst ",$leaf
                        $discoveryData.Add($classInstance)
                        if($subsystem.Name -eq 'Storage')
                        {
                            $storageServerRelationID = GetMonitoringRelation "Dell.SDK.Server.Storage.Relation"
                            CreateContainmentRelation $storageServerRelationID $rootInstance $classInstance
                        }
                    }
				}				
			}
            else{
                    $leaf = $subSystem.Name
                    $monitoringClass = GetMonitoringClass $classname 
                    $classInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
                    $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($rootKeyName),$leaf)
                    $extrComp = $className.Split(".")
                    $count = 1
                    if(($classname -eq "Dell.SDK.Server.ControllerSensor") -or ($classname -eq "Dell.SDK.Server.EnclosureSensor"))
                    {
                        $ControllerHash = $subTree  
                        $count = $ControllerHash.$leaf.Count
                    }
                    if($count -ne 0)
                    {
                        populateInventory -componentName $extrComp.get($extrComp.Length - 1) -MPKeyName $rootKeyName -MPKeyValue $leaf -classInstance $classInstance -monitoringClass $monitoringClass
                        $subsystemparent = storageMap -ParentKeyId $subsystemparent -subSystemClassName $classname -MPKeyName $rootKeyName -MPKeyValue $leaf -classInstance $classInstance
                        $discoveryData.Add($classInstance)
                    }

                }
            if($subSystem.Value -is [system.array])
            {    
				foreach($leaf in $subSystem.Value)
                {   
                    $leafNodeClassName = "Dell.SDK.Server."+$subSystem.Name
                    if(($rootParentClassName -eq 'Dell.SDK.Server.Controller') -and ($subSystem.Name -eq 'PhysicalDisk'))
                    {
                        $leafNodeClassName = "Dell.SDK.Server.DirectAttachedPhysicalDisk"
                    }
                    $rootKeyName = "Key"
                    $monitoringClass = GetMonitoringClass $leafNodeClassName 
                    $classInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
                    $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($rootKeyName),$leaf)
                    populateInventory -componentName $subSystem.Name -MPKeyName $rootKeyName -MPKeyValue $leaf -classInstance $classInstance -monitoringClass $monitoringClass
					$tmp = storageMap -ParentKeyId $subsystemparent -subSystemClassName $leafNodeClassName -MPKeyName $rootKeyName -MPKeyValue $leaf -classInstance $classInstance
                    $discoveryData.Add($classInstance)
                }
            }
            else
            {
				CreateSubsystemNodes $subSystem.value $rootKeyName $subSystem.Name $classname $treespacing $subsystemparent          
            }
        }
    }
}

function populateInventory{
    param($componentName, $MPKeyName, $MPKeyValue, $classInstance, $monitoringClass)
    $compList = $jsonObj.$componentName
    $currInstHash = $null
    foreach($comp in $compList)
    {
        if($comp.$MPKeyName -eq $MPKeyValue)
        {
            $currInstHash = $comp
        }
    }
    if($currInstHash)
    {
        foreach($attr in $monitoringClass.PropertyCollection.Name)
        {
            if($currInstHash.ContainsKey($attr))
            {              
                $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($attr), $currInstHash.$attr)
            }
        }
        
        if($monitoringClass.PropertyCollection.Name.Contains("Component"))
        {
            $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Component"),$ComponentName)
        }
		
		$displayProp = $entityClass.GetMonitoringProperty("DisplayName")
		if(($componentName -eq "License") -and ($currInstHash.ContainsKey('LicenseDescription'))){
			$classInstance.SetMonitoringPropertyValue($displayProp, $currInstHash.LicenseDescription)
		}
		elseif(($componentName -eq "HostNIC") -and ($currInstHash.ContainsKey('Name'))){
			$classInstance.SetMonitoringPropertyValue($displayProp, $currInstHash.Name)
		}
		elseif($componentName -eq "BIOS"){
			$classInstance.SetMonitoringPropertyValue($displayProp, $rootKey)
		}
		elseif($componentName -eq "System"){
			$classInstance.SetMonitoringPropertyValue($displayProp, $sysDisplayName)
            if($currInstHash.ContainsKey('iDRACURL'))
            {
                $ipAddress = $currInstHash.iDRACURL
                $parts = $ipAddress.Split("//")
                $RemoteAccessIP = $parts[2].Substring(0, $parts[2].IndexOf(":")) 
                $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("RemoteAccessIP"),$RemoteAccessIP)
            }
		}
		elseif(($componentName -eq "VFlash") -and ($currInstHash.ContainsKey('DeviceDescription'))){
			$classInstance.SetMonitoringPropertyValue($displayProp, $currInstHash.DeviceDescription)
		}
		elseif($componentName -eq "ControllerSensor"){
			$classInstance.SetMonitoringPropertyValue($displayProp, "Controller Sensor")
		}
		elseif($componentName -eq "EnclosureSensor"){
			$classInstance.SetMonitoringPropertyValue($displayProp, "Enclosure Sensor")
		}
        elseif(($componentName -match "Sensors_") -and ($currInstHash.ContainsKey('DeviceID'))){
            $classInstance.SetMonitoringPropertyValue($displayProp, $currInstHash.DeviceID)
        }
		else{
			$classInstance.SetMonitoringPropertyValue($displayProp, $currInstHash.Key)
		}
    }
}

function WriteToAppliance{
    param($Device, $passed, $failed)
    $creds = @{"scomdevices"=@{"PassedDevices"= $passed} + @{"FailedDevices"= $failed}}
	$jsonDeviceObj = $creds | ConvertTo-Json
    return $jsonDeviceObj
}

function CreateLogFile{
param($scriptname, $logDirectory)
try{
	    $ErrorActionPreference = "SilentlyContinue"
	    $TempFolder = $env:SystemRoot + "\Temp"
	    $LogLocation = $TempFolder + "\" + $logDirectory + "\"
	    If(!(Test-Path -path($TempFolder))){
		    # create the directory if not present
		    New-Item $TempFolder -type directory 
	    }
	    If(!(Test-Path -path($LogLocation))){
		    # create the directory if not present
		    New-Item $LogLocation -type directory 
	    }
	    $Global:LogFileLocation = $LogLocation + $scriptname + ".log"
	    If(!(Test-Path -path($LogFileLocation))){
		    # create the file if it does not exist
		    New-Item $LogFileLocation -type file
	    }
	    Else{
		    $logFileSize = Get-ChildItem $LogFileLocation | ForEach-Object {($_.Length/1KB)}
		    If ($logFileSize -gt 512){
			    # existingLogFile is greater than 512 KB
			    $archiveTime = Get-Date -f "yyyy-MM-dd_HH.mm" 
			    Rename-Item $LogFileLocation ("ArchivedLog_" + $scriptname + "_" + $archiveTime + ".log")
			    New-Item $LogFileLocation ChildItem-type file
		    }                    
	    }
    }
    catch{
	    $ErrorMessage = $_.Exception.Message
	    $FailedItem = $_.Exception.ItemName
	    $message = "Exception in creating log file. Exception-Message:" + $ErrorMessage + $FailedItem
	    $api.LogScriptEvent("DellSDKServerDiscovery.ps1",1345,0,$message)
    }

}

$discOverrides = Get-SCOMDiscovery -Name "Dell.Server.Discovery" | Get-SCOMOverride -ErrorAction SilentlyContinue
if($discOverrides -ne $null)
{
	foreach ($override in $discOverrides)
	{
		if($override.Parameter -eq "LogLevel")
		{
			$logLevel = $override.Value
		}
	}
}

foreach($serviceTag in $serviceTagFolders)
{
    $scriptname = $serviceTag + "_DellSDKServerDiscovery"
    $logDirectory = "DellSDKServer_log"	
    if($logLevel -eq 1){		
        CreateLogFile $scriptname $logDirectory }   
    try{
        $jsonpath = $nfspath+"\"+$serviceTag+"\Inventory.json"
        if ((Get-Item $jsonpath -ErrorAction Ignore) -ne $null) {
	        $jsonObj = ConvertFrom-Json -InputObject (Get-Content $jsonpath -Raw)
            $jsonObj = ConvertPSObjectToHashtable $jsonObj
        }

        $comptTreeFilePath = $nfspath+"\"+$serviceTag+"\CompTree.json"
        if ((Get-Item $comptTreeFilePath -ErrorAction Ignore) -ne $null) {
	        $comptTreeObj = ConvertFrom-Json -InputObject (Get-Content $comptTreeFilePath -Raw)
			$comptTreeObj = ConvertPSObjectToHashtable $comptTreeObj
        }
    }
    catch{
        $ErrorMessage = $_.Exception.Message
        $FailedItem = $_.InvocationInfo.ScriptLineNumber
        psDebugLog -level 1 -message  ("Exception occurred : Line - " + $FailedItem + " - " + $ErrorMessage )
        $failed += $serviceTag
        continue
    }
	$storageTreeMap = @{}
    $script:storageTreeIditer = 0
    $sysDisplayName = $serviceTag
    $rootParent = $comptTreeObj.System

    $comptTreeSysObj = $comptTreeObj.System
   
    $rootKey = $serviceTag
    $Device = $jsonObj.System."$("_Type")"
    $hostName = $jsonObj.System."$("HostName")"
    $systemGeneration = $jsonObj.System."$("SystemGeneration")"
    $model = $jsonObj.System."$("Model")"
    $manufacturer = $jsonObj.System."$("Manufacturer")"
    $targetComputer = $jsonObj.System."$("MachineName")"
    
    if($targetComputer -eq "Not Available")
    {
        $targetComputer = $computerName
    }

    if(($hostName -ne $null) -and ($hostName -ne " ") -and ($hostName -ne "Not Available"))
    {
        $sysDisplayName = $hostName
    }
    else
    {
        $ipAddress = $jsonObj.iDRAC."$("URLString")"
        $parts = $ipAddress.Split("//")
        $sysDisplayName = $parts[2].Substring(0, $parts[2].IndexOf(":"))
    }
    if($Device -eq "Server")
    {
        cleanUpInstances
        CreateRootNode $rootParent "System" $serviceTag
        cleanUpSubsystems
        $passed += $serviceTag
    }
    else
    {
		$failed += $serviceTag
	}
    #$storageTreeMap
    $discoveryData.Overwrite($Connector)
}

$devicesPassedFailed = WriteToAppliance $Device $passed $failed
return $devicesPassedFailed